#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#  Copyright (c) 2019.       Mike Herbert
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
import os
import tkinter as tk
from typing import Dict

from geofinder import UtilListboxFrame, GeoDB, Loc
from geofinder.CachedDictionary import CachedDictionary

GEOID_TOKEN = 1
PREFIX_TOKEN = 2


class SetupReplaceFrame(UtilListboxFrame.ListboxFrame):
    """
    SetupReplaceFrame is derived from ListboxFrame
    Display items in Global Replace List

    ListboxFrame Displays a scrolling list box that is based on a CachedDictionary.
    ListboxFrame defines the overall Grid Layout
    """

    def __init__(self, frame, title: str, dir_name: str, cache_filename: str):
        # Initialize GEO database
        self.geodb = GeoDB.GeoDB(os.path.join(dir_name, 'geodata.db'))

        # Read in dictionary listing output text replacements
        self.output_replace_cd = CachedDictionary(dir_name, "output_list.pkl")
        self.output_replace_cd.read()
        self.output_replace_dct: Dict[str, str] = self.output_replace_cd.dict

        super().__init__(frame, title, dir_name, cache_filename)
        self.tree.heading("#0", text="Original", anchor=tk.W)
        self.tree.heading("pre", text="Replacement", anchor=tk.W)

    def load_handler(self):
        # Load in list and display
        self.clear_display_list(self.tree)
        place = Loc.Loc()

        for item in sorted(self.dict):
            # get lat long
            replacement = self.dict[item]
            rep_token = replacement.split('@')
            if len(rep_token) < 2:
                self.logger.debug(f'blank item=[{item}] rep=[{replacement}]')
                continue
            place.target = rep_token[GEOID_TOKEN]
            self.geodb.lookup_geoid(place=place)

            if len(place.georow_list) > 0:
                # Copy geo row to Place
                self.geodb.copy_georow_to_place(row=place.georow_list[0], place=place)
                if 'oston' in item:
                    self.logger.debug(f'{item} id={rep_token[GEOID_TOKEN]} state={place.admin1_id}')
            else:
                if len(place.target) == 0:
                    place.clear()
                    place.city1 = f'<DELETE>'
                else:
                    place.clear()
                    place.city1 = f'Database error for {replacement}'
                place.place_type = Loc.PlaceType.CITY

            # Get prefix if there was one
            if len(rep_token) > 2:
                place.prefix = rep_token[PREFIX_TOKEN]

            nm = place.format_full_nm(self.output_replace_dct)
            if len(place.prefix) > 0:
                line = f'[{place.prefix}]{place.prefix_commas}{nm}'
            else:
                line = f'{nm}'

            self.list_insert(self.tree, item, line)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#  Copyright (c) 2019.       Mike Herbert
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
import logging
import sqlite3
import sys
from tkinter import messagebox

from geofinder.GeoKeys import Query, Result

DB_CORRUPT_MSG = 'Database error'


class DB:
    """
    Sqlite3  helper functions
    """

    def __init__(self, db_filename: str):
        self.logger = logging.getLogger(__name__)

        self.select_str = '*'
        self.order_str = ''
        self.limit_str = ''
        self.cur = None

        # create a database connection
        self.conn = self.connect(db_filename=db_filename)
        if self.conn is None:
            self.err = True
            self.logger.error(f"Error! cannot open database {db_filename}.")
            raise ValueError('Cannot open database')
        else:
            self.err = False
            self.conn.isolation_level = None

    def connect(self, db_filename: str):
        """ create a database connection to the SQLite database
            specified by db_file
        :param db_filename: database filename
        :return: Connection object or None
        """
        try:
            conn = sqlite3.connect(db_filename)
            self.logger.info(f'DB {db_filename} connected')
            return conn
        except Exception as e:
            messagebox.showwarning('Error', f'{DB_CORRUPT_MSG}\n {e}')
            self.err = True
            self.logger.error(e)
            sys.exit()

    def set_params(self, select_str: str, order_str: str, limit_str: str):
        # Set values for SELECT, ORDER BY, and LIMIT
        self.select_str = select_str
        self.order_str = order_str
        self.limit_str = limit_str

    def set_pragma(self, pragma: str):
        # Set PRAGMA e.g. temp_store = memory;
        cur = self.conn.cursor()
        cur.execute(pragma)
        self.conn.commit()

    def create_table(self, create_table_sql: str):
        """ create a table from the create_table_sql statement
        :param create_table_sql: a CREATE TABLE statement
        """
        try:
            c = self.conn.cursor()
            c.execute(create_table_sql)
            self.conn.commit()
            self.logger.info(f'Create DB table {create_table_sql[27:36]}')  # Lazy attempt to get table name for logging
        except Exception as e:
            messagebox.showwarning('Error', e)
            self.err = True
            self.logger.error(e)
            sys.exit()

    def create_index(self, create_table_sql: str):
        try:
            c = self.conn.cursor()
            c.execute(create_table_sql)
            self.conn.commit()
        except Exception as e:
            messagebox.showwarning('Error', e)
            self.err = True
            self.logger.error(e)
            sys.exit()

    def delete_table(self, tbl):
        cur = self.conn.cursor()
        # noinspection SqlWithoutWhere
        try:
            cur.execute(f'DELETE FROM {tbl}')
        except Exception as e:
            messagebox.showwarning('Error', f'{DB_CORRUPT_MSG}\n {e}')
            self.err = True
            self.logger.error(e)
            sys.exit()

    def get_row_count(self):
        cur = self.conn.cursor()
        cur.execute('SELECT COUNT(*) FROM main.geodata')
        res = cur.fetchall()
        count = res[0][0]
        return count

    def begin(self):
        self.cur = self.conn.cursor()
        self.cur.execute('BEGIN')

    def execute(self, sql, args):
        #try:
        if True:
            self.cur.execute(sql, args)
        """
        except Exception as e:
            messagebox.showwarning('Error', f'{DB_CORRUPT_MSG}\n {e}')
            self.err = True
            self.logger.error(e)
            sys.exit()
        """
        return self.cur.lastrowid

    def commit(self):
        # Commit transaction
        self.cur.execute("commit")

    def select(self, where, from_tbl, args):
        cur = self.conn.cursor()
        sql = f"SELECT {self.select_str} FROM {from_tbl} WHERE {where} {self.order_str} {self.limit_str}"
        res = [""]
        try:
            cur.execute(sql, args)
            res = cur.fetchall()
        except Exception as e:
            messagebox.showwarning('Error', f'{DB_CORRUPT_MSG}\n {e}')
            self.err = True
            self.logger.error(e)
            sys.exit()
        return res

    def db_test(self, from_tbl: str):
        where = 'name like ? AND country like ?'
        args = ('b%','b%')

        cur = self.conn.cursor()
        sql = f"SELECT {self.select_str} FROM {from_tbl} WHERE {where} {self.order_str} {self.limit_str}"

        self.logger.debug(f'db test sql={sql} args=[{args}]')
        try:
            cur.execute(sql, args)
            res = cur.fetchall()
            self.logger.debug('DB no error')
            return False
        except Exception as e:
            self.logger.debug('DB ERROR')
            return True

    def process_query_list(self, from_tbl: str, query_list: [Query]):
        # Try each query in list until we find a match
        row_list = None
        res = Result.NO_MATCH
        for query in query_list:
            row_list = self.select(query.where, from_tbl, query.args)
            #self.logger.debug(f'select x from {from_tbl}  where {query.where} val={query.args}')
            if len(row_list) > 0:
                res = query.result  # Set specified success code
                #self.logger.debug(row_list)
                # Found match.  Break out of loop
                break

        return row_list, res

    def set_speed_pragmas(self):
        # Set DB pragmas for speed.  These can lead to corruption!
        self.logger.info('Database pragmas set for speed')
        for txt in ['PRAGMA temp_store = memory',
                    'PRAGMA journal_mode = off',
                    'PRAGMA cache_size = -500',
                    'PRAGMA synchronous = 0']:
            self.set_pragma(txt)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#  Copyright (c) 2019.       Mike Herbert
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA

import logging
from tkinter import ttk
import tkinter as tk

from geofinder import UtilListboxFrame
from geofinder import TKHelper as Widge

default = []


class UtilOutputFilterFrame(UtilListboxFrame.ListboxFrame):
    """
    SetupFeatureList allows users to add or delete items in the Feature List
    The Feature list is the Feature types that we will load from a Geonames.org file.  For example, we will load
    Villages, Squares, Parks, Cemetaries, etc but not load Refineries, Glaciers, and Coral Reefs.
    For details on Feature Types see http://www.geonames.org/export/codes.html
    SetupFeatureList is derived from ListboxFrame:

    ListboxFrame Displays a scrolling list box that is based on a CachedDictionary.
    ListboxFrame defines the overall Grid Layout
    """


    def __init__(self, frame, title, dir_name, cache_filename):
        self.logger = logging.getLogger(__name__)

        # Add these in addition to the standard widgets we inherit from ListBoxFrame
        self.add_button = ttk.Button(frame, text="add", command=self.add_handler, width=UtilListboxFrame.BUTTON_WIDTH)
        self.add_label = Widge.CLabel(frame, text="Enter replacements below and click on Add", style='Info.TLabel')
        self.add_label2 = Widge.CLabel(frame, text="Original:", style='Info.TLabel')
        self.add_entry: Widge.CEntry = Widge.CEntry(frame, text=" orig  ", width=15)  # , style='Info.TLabel')
        self.add_replace: Widge.CEntry = Widge.CEntry(frame, text=" rr  ", width=15)  # , style='Info.TLabel')
        self.add_label3 = Widge.CLabel(frame, text="Replacement:", style='Info.TLabel')

        super().__init__(frame, title, dir_name, cache_filename)

        self.tree.heading("#0", text="Original", anchor=tk.W)
        self.tree.heading("pre", text="Replacement", anchor=tk.W)

        # If dictionary is empty, load in defaults
        if len(self.dict) == 0:
            self.logger.error('Output filter list is empty. loading defaults')
            self.set_default(default)
            self.load_defaults()
            super().add_handler()

    def configure_widgets(self, frm):
        # Add the standard widgets in ListBoxFrame
        super().configure_widgets(frm)

        # Add these in addition to the standard widgets in ListBoxFrame
        # todo - remove set_grid_position and just use element.grid()
        Widge.TKHelper.set_grid_position(self.add_button, "country_button", grd=self.grd)
        Widge.TKHelper.set_grid_position(self.add_label, "country_label", grd=self.grd)
        Widge.TKHelper.set_grid_position(self.add_entry, "country_entry", grd=self.grd)
        Widge.TKHelper.set_grid_position(self.add_replace, "country_entry2", grd=self.grd)
        Widge.TKHelper.set_grid_position(self.add_label2, "country_label2", grd=self.grd)
        Widge.TKHelper.set_grid_position(self.add_label3, "country_label3", grd=self.grd)

    def add_handler(self):
        # Allow user to add an item to list.
        val: str = self.add_entry.get_text()
        val2: str = self.add_replace.get_text()
        self.add_label.configure(style='Info.TLabel')
        self.dict[val] = val2  # Add item to dict
        super().add_handler()

    def load_handler(self):
        # Load in list and display
        self.clear_display_list(self.tree)

        for item in sorted(self.dict):
            self.list_insert(self.tree, item, self.dict[item])



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#  Copyright (c) 2019.       Mike Herbert
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
import logging
import os
from typing import Union, Tuple

from geofinder import Progress


class AncestryFile:
    """
    Base class for Gedcom file handler and Gramps XML file handler
    Basic routines to Read/Parse and Write Ancestry files focused on place entries.
    Scan - Read through  file, find specified Tag entry.
    Write out all other entries as-is if out_path is not None
    """

    def __init__(self, in_path: str, out_suffix: str, cache_d, progress: Union[None, Progress.Progress]):
        self.build = False
        self.logger = logging.getLogger(__name__)
        self.progress_bar = progress
        self.output_latlon = True
        self.filesize = 0
        self.infile = None
        self.error = False
        self.out_path = in_path + '.' + out_suffix
        self.more_available = False

        self.place_total = 0
        self.line_num = 0

        self.value: str = ""
        self.tag: str = ""        # PLAC indicates this is a Place entry

        # GEDCOM  meta data
        self.id: str = ''
        self.name: str = ""
        self.event_year: int = 0
        self.event_name: str = ""
        self.date = ''
        self.abt_flag = False

        if out_suffix is not None:
            # Create an output file with same name with "out.ged" appended
            self.outfile = open(self.out_path, "w",
                                encoding='utf-8')
        else:
            self.outfile = None

        # Open Ancestry file in utf-8.  Replace any non-UTF-8 characters (e.g. Latin)
        err = self.open(in_path)
        if err:
            return

        if self.output_latlon is False:
            self.logger.warning('### OUTPUT OF LAT/LON IS DISABLED ###')

    def open(self, in_path) -> bool:
        # Open ancestry file
        if os.path.exists(in_path):
            self.infile = open(in_path, 'rU', encoding='utf-8', errors='replace')
            self.filesize: int = int(os.path.getsize(in_path))  # Used for progress bar calculation
            self.logger.info(f'Opened  {in_path} Size={self.filesize}')
            self.error: bool = False
        else:
            self.logger.error(f"File {in_path} not found")
            self.error = True
        return self.error

    def get_next_place(self) -> (str, bool):
        # Scan  file for Place entry or EOF
        # Output all other lines as-is to outfile
        while True:
            line, err = self.read_and_parse_line()
            if err:
                return '', True  # End of file reached

            if self.tag == 'PLAC':
                # Found the target line.  Break out of loop
                entry = self.value
                if entry is None:
                    continue
                return entry, False
            if self.tag == 'IGNORE':
                pass
            else:
                # Not a target entry.   Write out line as-is
                if self.outfile is not None:
                    self.outfile.write(line)

    def read_and_parse_line(self) -> Tuple[str, bool]:
        # Read a line from file.  Handle line.
        if not self.more_available:
            line = self.infile.readline()
            self.line_num += 1
            if line == "":
                # End of File
                return "", True
        else:
            line = ''

        # Separate the line into  parts
        self.parse_line(line)

        #  Keep track of  lines for this event so we have full view of event
        self.collect_event_details()

        return line, False

    def collect_event_details(self):
        """ Collect details for event - last name, event date, and tag in GEDCOM file."""
        pass

    def peak_next_line(self):
        """ Return a peak at next line but dont move forward in file """
        pos = self.infile.tell()
        line = self.infile.readline()
        self.infile.seek(pos)  # Back up to where we were
        return line

    def close(self):
        self.infile.close()
        if self.outfile is not None:
            self.outfile.close()

    def progress(self, msg: str, percent: int):
        """ Display progress update """
        if percent < 2:
            percent = 2
        if self.progress_bar is not None:
            self.progress_bar.update_progress(percent, msg)
        else:
            self.logger.debug('prog is None')

    def get_name(self, nam: str, depth: int = 0) -> str:
        return ''

    # Derived classes must override these:

    def parse_line(self, line: str):
        # Called by read_and_parse_line for each line in file
        # return each entry in self.value with self.tag set to PLAC
        pass

    def write_updated(self, txt):
        """ Write updated place out """
        pass

    def write_asis(self):
        """ Write out place entry as is.  """
        pass

    def write_lat_lon(self, lat: float, lon: float):
        """ Write out a GEDCOM PLACE MAP entry with latitude and longitude. """
        pass



# !/usr/bin/env python3
# -*- coding: utf-8 -*-
import logging
import tkinter as tk
from tkinter import *


class Tooltip:
    """
    It creates a tooltip for a given widget as the mouse goes on it.

    see:

    https://stackoverflow.com/questions/3221956/
           what-is-the-simplest-way-to-make-tooltips-
           in-tkinter/36221216#36221216

    http://www.daniweb.com/programming/software-development/
           code/484591/a-tooltip-class-for-tkinter

    - Originally written by vegaseat on 2014.09.09.

    - Modified to include a delay time by Victor Zaccardo on 2016.03.25.

    - Modified
        - to correct extreme right and extreme bottom behavior,
        - to stay inside the screen whenever the tooltip might go out on
          the top but still the screen is higher than the tooltip,
        - to use the more flexible mouse positioning,
        - to add customizable background color, padding, waittime and
          wraplength on creation
      by Alberto Vassena on 2016.11.05.

      Tested on Ubuntu 16.04/16.10, running Python 3.5.2

    """

    def __init__(self, root, widget,
                 *,
                 bg='#FFFFEA',
                 pad=(5, 3, 5, 3),
                 text='widget info',
                 waittime=700,
                 wraplength=250):
        self.logger = logging.getLogger(__name__)
        self.waittime = waittime  # in miliseconds, originally 500
        self.wraplength = wraplength  # in pixels, originally 180
        self.widget = widget
        self.text = text
        #self.widget.bind("<Enter>", self.onEnter)
        #self.widget.bind("<Leave>", self.onLeave)
        #self.widget.bind("<ButtonPress>", self.onLeave)
        self.bg = bg
        self.pad = pad
        self.id = None
        self.tw = None

    def onEnter(self, event):
        # self.logger.debug(f'Enter {self.text}')
        self.schedule()

    def onLeave(self, event):
        # self.logger.debug(f'Leave {self.text}')
        self.unschedule()
        self.hide()

    def schedule(self):
        self.unschedule()
        self.id = self.widget.after(self.waittime, self.show)

    def unschedule(self):
        id_ = self.id
        self.id = None
        if id_:
            self.widget.after_cancel(id_)

    def show(self):
        def tip_pos_calculator(widget, label,
                               *,
                               tip_delta=(10, 5), pad=(5, 3, 5, 3)):

            w = widget

            s_width, s_height = w.winfo_screenwidth(), w.winfo_screenheight()

            width, height = (pad[0] + label.winfo_reqwidth() + pad[2],
                             pad[1] + label.winfo_reqheight() + pad[3])

            mouse_x, mouse_y = w.winfo_pointerxy()

            x1, y1 = mouse_x + tip_delta[0], mouse_y + tip_delta[1]
            x2, y2 = x1 + width, y1 + height

            x_delta = x2 - s_width
            if x_delta < 0:
                x_delta = 0
            y_delta = y2 - s_height
            if y_delta < 0:
                y_delta = 0

            offscreen = (x_delta, y_delta) != (0, 0)

            if offscreen:

                if x_delta:
                    x1 = mouse_x - tip_delta[0] - width

                if y_delta:
                    y1 = mouse_y - tip_delta[1] - height

            offscreen_again = y1 < 0  # out on the top

            if offscreen_again:
                # No further checks will be done.

                # TIP:
                # A further mod might automagically augment the
                # wraplength when the tooltip is too high to be
                # kept inside the screen.
                y1 = 0

            return x1, y1

        bg = self.bg
        pad = self.pad
        widget = self.widget

        # creates a toplevel window
        self.tw = Toplevel(widget)

        # Leaves only the label and removes the app window
        # self.tw.wm_overrideredirect(True)
        # self.tw.wm_attributes('-type', 'splash')
        # self.tw.lift()

        win = tk.Frame(self.tw,
                       background=bg,
                       borderwidth=0)
        label = tk.Label(win,
                         text=self.text,
                         justify=tk.LEFT,
                         background=bg,
                         relief=tk.SOLID,
                         borderwidth=0,
                         wraplength=self.wraplength)

        label.grid(padx=(pad[0], pad[2]),
                   pady=(pad[1], pad[3]),
                   sticky=tk.NSEW)
        win.grid()

        x, y = tip_pos_calculator(widget, label)

        self.tw.wm_geometry("+%d+%d" % (x, y))
        # self.logger.debug(f'Display {self.text}')
        win.update_idletasks()

    def hide(self):
        tw = self.tw
        if tw:
            tw.destroy()
        self.tw = None



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#  Copyright (c) 2019.       Mike Herbert
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA

import logging
import tkinter as tk
from tkinter import *
from tkinter import ttk
from tkinter.ttk import *

from geofinder import Country, GeodataFiles, UtilListboxFrame, AppStyle
from geofinder.TKHelper import TKHelper


class SetupCountriesFrame(UtilListboxFrame.ListboxFrame):
    """
    ListboxAdd is derived from ListboxFrame
    Supports country lists
    Adds support for adding to the list by typing in a new entry

    ListboxFrame Displays a scrolling list box that is based on a CachedDictionary.
    ListboxFrame defines the overall Grid Layout
    """

    def __init__(self, frame, title: str, dir_name: str, cache_dir: str, cache_filename: str):
        self.logger = logging.getLogger(__name__)
        self.logger.debug(f'SetupConfigureCountries dir {dir_name} cache dir {cache_dir} file {cache_filename}')
        self.country_dict = {}

        self.pad = ttk.Label(frame, text="", style='Info.TLabel')  # blank padding row

        self.add_label = ttk.Label(frame, text="All Countries - Select countries below to add to supported list and click Add", style='Info.TLabel')
        self.add_button = ttk.Button(frame, text="add", command=self.add_handler, width=UtilListboxFrame.BUTTON_WIDTH)
        self.scrollbar2 = Scrollbar(frame)

        self.listbox_all_countries = ttk.Treeview(frame, style="Plain.Treeview")  # , selectmode="browse")
        self.listbox_all_countries.tag_configure('odd', background=AppStyle.ODD_ROW_COLOR)
        self.listbox_all_countries.tag_configure('even', background='white')

        self.listbox_all_countries["columns"] = ("pre",)
        self.listbox_all_countries.column("#0", width=400, minwidth=100, stretch=tk.NO)
        self.listbox_all_countries.column("pre", width=500, minwidth=50, stretch=tk.NO)
        self.listbox_all_countries.heading("#0", text="Name", anchor=tk.W)
        self.listbox_all_countries.heading("pre", text="Code", anchor=tk.W)

        self.listbox_all_countries.config(yscrollcommand=self.scrollbar2.set)
        self.scrollbar2.config(command=self.listbox_all_countries.yview)

        super().__init__(frame, title, cache_dir, cache_filename)
        self.tree.heading("#0", text="Name", anchor=tk.W)
        self.tree.heading("pre", text="Code", anchor=tk.W)

        self.logger.info('geodatafiles loading')
        self.geoFiles = GeodataFiles.GeodataFiles(dir_name, None)
        self.logger.info('  geodatafiles initialized')

        self.load_handler_all()

    def configure_widgets(self, frm):
        super().configure_widgets(frm)

        # Add Lable for allow user to Add to list
        TKHelper.set_grid_position(self.pad, "pad", grd=self.grd)

        # Add Lable for allow user to Add to list
        TKHelper.set_grid_position(self.add_label, "add_label", grd=self.grd)

        # Add Button to allow user to Add to list
        TKHelper.set_grid_position(self.add_button, "add_button", grd=self.grd)

        # Create listbox of countries to add with scrollbar        
        TKHelper.set_grid_position(self.listbox_all_countries, "listbox_all_countries", grd=self.grd)
        self.scrollbar2.config(command=self.listbox_all_countries.yview)
        TKHelper.set_grid_position(self.scrollbar2, "scrollbar2", grd=self.grd)

    def load_handler(self):
        # Load in list and display - Need to Reverse the Key and Val
        self.clear_display_list(self.tree)

        if len(self.dict)==0:
            # Country list is empty.  Hide Remove button and text
            self.remove_button.grid_remove()
            self.status.set_text('')
        else:
            self.remove_button.grid()
            self.status.set_text('Highlight items above to remove and click Remove.')

        for key in sorted(self.dict):
            if len(self.dict[key]) > 1:
                self.list_insert(self.tree, f"{self.dict[key]}", f"{key}")
            else:
                self.list_insert(self.tree, f"{key}", '')

    def delete_items(self, tree, dct):
        # Delete any items in the list that the user selected
        items = tree.selection()
        for line in items:
            col1 = self.tree.item(line, "text")
            col2 = self.tree.item(line, 'values')[0]
            dct.pop(col2, None)

        self.load_handler()  # Reload display

    def load_handler_all(self):
        # Load in list of all countries and display
        # self.listbox_all_countries.delete(0, END)
        self.clear_display_list(self.listbox_all_countries)
        self.logger.info('Building country list: {} countries'.format(len(Country.country_dict)))
        for name in sorted(Country.country_dict):
            row = Country.country_dict[name]
            self.list_insert(self.listbox_all_countries, name.lower(), row[0].lower())

    def add_handler(self):
        # Add items user selected to supported list
        # Delete any items in the list that the user selected

        items = self.listbox_all_countries.selection()
        for line in items:
            col1 = self.listbox_all_countries.item(line, "text")
            col2 = self.listbox_all_countries.item(line, 'values')[0]
            print(f"ADD [{col1}]  [{col2}]")

            if len(col2) > 0:
                self.dict[col2.strip(" ")] = col1.strip(" ")  # Add key for entry to dict
            else:
                self.dict[col1.strip(" ")] = " "  # Add key for entry to dict

        super().add_handler()



import argparse

from geofinder.Loc import PlaceType

from geofinder import GeoKeys


def filter(place_name):
    # Separate out arguments
    tokens = place_name.split(",")
    args = []
    for tkn in tokens:
        if '--' in tkn:
            args.append(tkn.strip(' '))

    # Parse options in place name
    parser = argparse.ArgumentParser(description="Parses command.")
    parser.add_argument("-f", "--feature", help="Feature code, PPL etc.")
    parser.add_argument("-i", "--iso", help="ISO country code.")
    options = parser.parse_args(args)

    city1 = GeoKeys.normalize(tokens[0])
    target = city1
    country_iso = options.iso.lower()
    feature = options.feature.upper()
    place_type = PlaceType.FILTER

    print(f'iso=[{country_iso}] feature=[{feature}]')

filter(place_name="Edinburgh, --feature='CH',--iso='GB'")